<?xml version="1.0" encoding="UTF-8"?>
<xsl:stylesheet version="1.0" xmlns:xsl="http://www.w3.org/1999/XSL/Transform" xmlns:exslt="http://exslt.org/common"
    xmlns:str="http://exslt.org/strings" xmlns:set="http://exslt.org/sets" exclude-result-prefixes="exslt str set">
    <!-- MAIN TEMPLATE -->
    <xsl:template match="/">
        <schema name="index" version="1.5">
            <!-- Prepare all field types declared in input -->
            <xsl:variable name="v_types">
                <xsl:for-each select="/schema/fields/field">
                    <type>
                        <xsl:call-template name="create-type" />
                    </type>
                </xsl:for-each>
            </xsl:variable>

            <!-- SOLR TYPES -->
            <types>
                <!-- Regular field types -->
                <fieldType name="untokenized" class="solr.TextField" sortMissingLast="true" positionIncrementGap="1">
                    <analyzer>
                        <tokenizer class="solr.KeywordTokenizerFactory" />
                    </analyzer>
                </fieldType>
                <fieldType name="long" class="solr.TrieLongField" precisionStep="0" omitNorms="false" positionIncrementGap="1" />
                <fieldType name="date" class="solr.TrieDateField" sortMissingLast="true" omitNorms="false" precisionStep="0"
                    positionIncrementGap="1" />
                <fieldType name="tdate" class="solr.TrieDateField" sortMissingLast="true" omitNorms="false" precisionStep="6"
                    positionIncrementGap="1" />
                <!-- for mathematics storing -->
                <fieldType name="math" class="solr.TextField" sortMissingLast="true" positionIncrementGap="1">
                    <analyzer>
                        <tokenizer class="solr.KeywordTokenizerFactory" />
                    </analyzer>
                </fieldType>

                <!-- Dynamically created field types. Loop over all field types defined (distinct): -->
                <xsl:for-each select="set:distinct(exslt:node-set($v_types)/type)">
                    <xsl:call-template name="generate-solr-type-field" />
                </xsl:for-each>
            </types>

            <!-- SOLR FIELDS -->
            <fields>
                <!-- Internal ID -->
                <field name="__ID__" type="untokenized" indexed="true" stored="true" required="true" />
                <field name="_version_" type="long" indexed="true" stored="true"/>
                <!-- Regular fields -->
                <xsl:for-each select="/schema/fields/field">
                    <xsl:call-template name="generate-solr-field" />
                </xsl:for-each>
            </fields>

            <!-- The rest of Solr configuration -->
            <uniqueKey>
                <xsl:text>__ID__</xsl:text>
            </uniqueKey>
            <defaultSearchField>__ID__</defaultSearchField>
            <solrQueryParser defaultOperator="OR" />
        </schema>
    </xsl:template>

    <!-- Generate Solr field declaration(s) for one field definition -->
    <xsl:template name="generate-solr-field">
        <!-- Get field template name declaration. -->
        <xsl:variable name="v_template" select="@template" />
        <!-- Get field content type -->
        <xsl:variable name="v_contentType">
            <xsl:choose>
                <xsl:when test="type">
                    <xsl:value-of select="type/@value" />
                </xsl:when>
                <xsl:otherwise>
                    <xsl:value-of select="/schema/templates/template[@name = $v_template]/type/@value" />
                </xsl:otherwise>
            </xsl:choose>
        </xsl:variable>
        <!-- Generate Solr type name. -->
        <xsl:variable name="v_type">
            <xsl:choose>
                <xsl:when test="$v_contentType = 'number'">
                    <xsl:text>long</xsl:text>
                </xsl:when>
                <xsl:when test="$v_contentType = 'date'">
                    <xsl:text>tdate</xsl:text>
                </xsl:when>
                <xsl:when test="$v_contentType = 'math'">
                    <xsl:text>math</xsl:text>
                </xsl:when>
                <xsl:when test="$v_contentType = 'text'">
                    <xsl:call-template name="create-type" />
                </xsl:when>
                <xsl:otherwise>
                    <xsl:message terminate="yes">
                        <xsl:text>Field type not supported: </xsl:text>
                        <xsl:value-of select="$v_contentType" />
                    </xsl:message>
                </xsl:otherwise>
            </xsl:choose>
        </xsl:variable>
        <!-- Set other field parameters -->
        <xsl:variable name="v_stored" select="stored or /schema/templates/template[@name = $v_template]/stored" />
        <xsl:variable name="v_termVectors" select="term-vectors or /schema/templates/template[@name = $v_template]/term-vectors" />
        <xsl:variable name="v_indexed" select="not (unindexed or /schema/templates/template[@name = $v_template]/unindexed)" />
        <xsl:variable name="v_multi-valued" select="not (single-valued or /schema/templates/template[@name = $v_template]/single-valued)" />

        <!-- Write Solr field declaration -->
        <xsl:choose>
            <!-- Dynamic suffix field -->
            <xsl:when test="dynamic-suffix">
                <dynamicField name="{@name}*" type="{$v_type}" stored="{$v_stored}" termVectors="{$v_termVectors}" indexed="{$v_indexed}"
                    multiValued="{$v_multi-valued}" />
                <!-- If EXACT_SEARCHABLE, then write out field declaration with special EXACT_SEARCH___ prefix -->
                <xsl:if test="exact-searchable or /schema/templates/template[@name = $v_template]/exact-searchable">
                    <dynamicField name="EXACT_SEARCH___{@name}*" type="{$v_type}" stored="false" termVectors="{$v_termVectors}"
                        indexed="{$v_indexed}" multiValued="{$v_multi-valued}" />
                </xsl:if>
                <!-- Untokenized copy -->
                <xsl:if test="untokenized-copy or /schema/templates/template[@name = $v_template]/untokenized-copy">
                    <dynamicField name="UNTOKENIZED___{@name}*" type="untokenized" stored="{$v_stored}" termVectors="{$v_termVectors}"
                        indexed="{$v_indexed}" multiValued="{$v_multi-valued}" />
                </xsl:if>
                <!-- If SORTABLE and of TEXT type, then write out field declaration with special SORT___ prefix -->
                <xsl:if test="(sortable or /schema/templates/template[@name = $v_template]/sortable) and $v_contentType = 'text'">
                    <dynamicField name="SORT___{@name}*" type="untokenized" stored="false" termVectors="{$v_termVectors}"
                        indexed="true" multiValued="false" />
                </xsl:if>
            </xsl:when>
            <!-- Regular field -->
            <xsl:otherwise>
                <field name="{@name}" type="{$v_type}" stored="{$v_stored}" termVectors="{$v_termVectors}" indexed="{$v_indexed}"
                    multiValued="{$v_multi-valued}" />
                <!-- If EXACT_SEARCHABLE, then write out field declaration with special EXACT_SEARCH___ prefix -->
                <xsl:if test="exact-searchable or /schema/templates/template[@name = $v_template]/exact-searchable">
                    <field name="EXACT_SEARCH___{@name}" type="{$v_type}" stored="false" termVectors="{$v_termVectors}" indexed="{$v_indexed}"
                        multiValued="{$v_multi-valued}" />
                </xsl:if>
                <!-- Untokenized copy -->
                <xsl:if test="untokenized-copy or /schema/templates/template[@name = $v_template]/untokenized-copy">
                    <field name="UNTOKENIZED___{@name}" type="untokenized" stored="{$v_stored}" termVectors="{$v_termVectors}"
                        indexed="{$v_indexed}" multiValued="{$v_multi-valued}" />
                </xsl:if>
                <!-- If SORTABLE and of TEXT type, then write out field declaration with special SORT___ prefix -->
                <xsl:if test="(sortable or /schema/templates/template[@name = $v_template]/sortable) and $v_contentType = 'text'">
                    <field name="SORT___{@name}" type="untokenized" stored="false" termVectors="{$v_termVectors}" indexed="true"
                        multiValued="false" />
                </xsl:if>
            </xsl:otherwise>
        </xsl:choose>
    </xsl:template>

    <!-- Template for Solr fieldType element creation. Based on current type name. -->
    <xsl:template name="generate-solr-type-field">
        <!-- Currently processing type -->
        <xsl:variable name="v_currentType" select="." />
        <!-- Split current type name back into parameters. (example: tokenized_positionIncrementGap_100 -> tokenized, positionIncrementGap, 100) -->
        <xsl:variable name="v_params" select="str:split($v_currentType,' AND ')" />

        <!-- untokenized with parameters -->
        <xsl:if test="(not ($v_params='tokenized')) and ($v_params = 'replace')">
            <fieldType name="{$v_currentType}" class="solr.TextField" sortMissingLast="true" positionIncrementGap="1">
                <analyzer>
                    <charFilter class="pl.edu.icm.yadda.search.solr.filter.PatternReplaceCharFilterFactory" pattern="{$v_params[1]}"
                        replacement="{$v_params[3]}" />
                    <tokenizer class="solr.KeywordTokenizerFactory" />
                </analyzer>
            </fieldType>
        </xsl:if>

        <!-- Tokenized types. (If $v_params contains 'tokenized'...) -->
        <xsl:if test="$v_params = 'tokenized'">
            <!-- Get span value (positionIncrementGap). If no defined -> set 1. -->
            <xsl:variable name="v_span">
                <xsl:choose>
                    <xsl:when test="$v_params = 'incrementGap'">
                        <!-- We expect span value to be at the end. -->
                        <xsl:value-of select="$v_params[count($v_params)]" />
                    </xsl:when>
                    <xsl:otherwise>
                        <xsl:text>1</xsl:text>
                    </xsl:otherwise>
                </xsl:choose>
            </xsl:variable>

            <!-- Write out Solr fieldType. -->
            <fieldType name="{$v_currentType}" class="solr.TextField" sortMissingLast="true" positionIncrementGap="{$v_span}">
                <analyzer type="query">
                    <xsl:if test="$v_params = 'replace'">
                        <charFilter class="pl.edu.icm.yadda.search.solr.filter.PatternReplaceCharFilterFactory" pattern="{$v_params[1]}"
                            replacement="{$v_params[3]}" />
                    </xsl:if>
                    <xsl:choose>
                        <!-- Whitespace tokenization -->
                        <xsl:when test="$v_params = 'whitespaceTokens'">
                            <tokenizer class="solr.WhitespaceTokenizerFactory" />
                        </xsl:when>
                        <!-- Simple tokenization -->
                        <xsl:when test="$v_params = 'simpleTokens'">
                            <xsl:if test="not($v_params = 'indexXml')">
                                <charFilter class="solr.HTMLStripCharFilterFactory" />
                            </xsl:if>
                            <tokenizer class="solr.LowerCaseTokenizerFactory" />
                        </xsl:when>
                        <!-- Regular tokenization -->
                        <xsl:otherwise>
                            <xsl:if test="not($v_params = 'indexXml')">
                                <charFilter class="solr.HTMLStripCharFilterFactory" />
                            </xsl:if>
                            <tokenizer class="solr.StandardTokenizerFactory" />
                            <!-- If keepStopwords, then we are NOT using solr.StopFilterFactory. -->
                            <xsl:if test="not($v_params = 'keepStopwords')">
                                <filter class="solr.StopFilterFactory" ignoreCase="true" words="stopwords.txt"
                                    enablePositionIncrements="true" />
                            </xsl:if>
                            <filter class="solr.ASCIIFoldingFilterFactory" />
                            <filter class="solr.StandardFilterFactory" />
                            <filter class="solr.LowerCaseFilterFactory" />
                        </xsl:otherwise>
                    </xsl:choose>
                </analyzer>
                 <analyzer type="index">
                    <xsl:if test="$v_params = 'replace'">
                        <charFilter class="pl.edu.icm.yadda.search.solr.filter.PatternReplaceCharFilterFactory" pattern="{$v_params[1]}"
                            replacement="{$v_params[3]}" />
                    </xsl:if>
                    <xsl:choose>
                        <!-- Whitespace tokenization -->
                        <xsl:when test="$v_params = 'whitespaceTokens'">
                            <tokenizer class="solr.WhitespaceTokenizerFactory" />
                            <xsl:if test="$v_params = 'consumeFirstToken'">
                                <filter class="pl.edu.icm.yadda.search.solr.filter.ConsumeTokenFilterFactory" firstTokensCount="1" />
                            </xsl:if>
                        </xsl:when>
                        <!-- Simple tokenization -->
                        <xsl:when test="$v_params = 'simpleTokens'">
                            <xsl:if test="not($v_params = 'indexXml')">
                                <charFilter class="solr.HTMLStripCharFilterFactory" />
                            </xsl:if>
                            <tokenizer class="solr.LowerCaseTokenizerFactory" />
                            <xsl:if test="$v_params = 'consumeFirstToken'">
                                <filter class="pl.edu.icm.yadda.search.solr.filter.ConsumeTokenFilterFactory" firstTokensCount="1" />
                            </xsl:if>
                        </xsl:when>
                        <!-- Regular tokenization -->
                        <xsl:otherwise>
                            <xsl:if test="not($v_params = 'indexXml')">
                                <charFilter class="solr.HTMLStripCharFilterFactory" />
                            </xsl:if>
                            <tokenizer class="solr.StandardTokenizerFactory" />
                            <xsl:if test="$v_params = 'consumeFirstToken'">
                                <filter class="pl.edu.icm.yadda.search.solr.filter.ConsumeTokenFilterFactory" firstTokensCount="1" />
                            </xsl:if>
                            <!-- If keepStopwords, then we are NOT using solr.StopFilterFactory. -->
                            <xsl:if test="not($v_params = 'keepStopwords')">
                                <filter class="solr.StopFilterFactory" ignoreCase="true" words="stopwords.txt"
                                    enablePositionIncrements="true" />
                            </xsl:if>
                            <filter class="solr.ASCIIFoldingFilterFactory" />
                            <filter class="solr.StandardFilterFactory" />
                            <filter class="solr.LowerCaseFilterFactory" />
                        </xsl:otherwise>
                    </xsl:choose>
                </analyzer>
            </fieldType>
        </xsl:if>
    </xsl:template>

    <!-- Template used to generate Solr field type name based on field parameters. (For example: TOKENIZED, KEEP_STOPWORDS -> tokenized_keepStopwords) -->
    <xsl:template name="create-type">
        <xsl:variable name="v_template" select="@template" />
        <xsl:if test="replace or /schema/templates/template[@name = $v_template]/replace">
            <xsl:choose>
                <xsl:when test="replace">
                    <xsl:value-of select="replace/@pattern" />
                </xsl:when>
                <xsl:otherwise>
                    <xsl:value-of select="/schema/templates/template[@name = $v_template]/replace/@pattern" />
                </xsl:otherwise>
            </xsl:choose>
            <xsl:text> AND replace AND </xsl:text>
            <xsl:choose>
                <xsl:when test="replace">
                    <xsl:value-of select="replace/@replacement" />
                </xsl:when>
                <xsl:otherwise>
                    <xsl:value-of select="/schema/templates/template[@name = $v_template]/replace/@replacement" />
                </xsl:otherwise>
            </xsl:choose>
            <xsl:text> AND </xsl:text>
        </xsl:if>
        <xsl:choose>
            <xsl:when test="tokenized or /schema/templates/template[@name = $v_template]/tokenized">
                <xsl:if test="tokenized or /schema/templates/template[@name = $v_template]/tokenized">
                    <xsl:text>tokenized</xsl:text>
                </xsl:if>
                <xsl:if test="simple-tokens or /schema/templates/template[@name = $v_template]/simple-tokens">
                    <xsl:text> AND simpleTokens</xsl:text>
                </xsl:if>
                <xsl:if test="whitespace-tokens or /schema/templates/template[@name = $v_template]/whitespace-tokens">
                    <xsl:text> AND whitespaceTokens</xsl:text>
                </xsl:if>
                <xsl:if test="index-xml or /schema/templates/template[@name = $v_template]/index-xml">
                    <xsl:text> AND indexXml</xsl:text>
                </xsl:if>
                <xsl:if test="consume-first-token or /schema/templates/template[@name = $v_template]/consume-first-token">
                    <xsl:text> AND consumeFirstToken</xsl:text>
                </xsl:if>
                <xsl:if test="keep-stopwords or /schema/templates/template[@name = $v_template]/keep-stopwords">
                    <xsl:text> AND keepStopwords</xsl:text>
                </xsl:if>
                <xsl:if test="span or /schema/templates/template[@name = $v_template]/span">
                    <xsl:text> AND incrementGap AND </xsl:text>
                    <xsl:choose>
                        <xsl:when test="span">
                            <xsl:value-of select="span/@value" />
                        </xsl:when>
                        <xsl:otherwise>
                            <xsl:value-of select="/schema/templates/template[@name = $v_template]/span/@value" />
                        </xsl:otherwise>
                    </xsl:choose>
                </xsl:if>
            </xsl:when>
            <xsl:otherwise>
                <xsl:text>untokenized</xsl:text>
            </xsl:otherwise>
        </xsl:choose>
    </xsl:template>
</xsl:stylesheet>